package mapx

import (
	"testing"

	"github.com/hsiafan/glow/v2/container/set"
	"github.com/hsiafan/glow/v2/container/slicex"
	"github.com/hsiafan/glow/v2/mathx/intx"
	"github.com/stretchr/testify/assert"
)

func TestGetOrDefault(t *testing.T) {
	m := map[int]int{
		1: 2,
		2: 4,
		3: 6,
	}
	assert.Equal(t, 2, Get(m, 1).OrElse(-1))
	assert.Equal(t, -1, Get(m, 4).OrElse(-1))
}

func TestPutIfAbsent(t *testing.T) {
	m := map[int]int{
		1: 2,
		2: 4,
		3: 6,
	}
	assert.Equal(t, 2, PutIfAbsent(m, 1, -1))
	assert.Equal(t, 2, m[1])
	assert.Equal(t, -1, PutIfAbsent(m, 4, -1))
	assert.Equal(t, -1, m[4])
}

func TestComputeIfAbsent(t *testing.T) {
	m := map[int]int{
		1: 2,
		2: 4,
		3: 6,
	}
	assert.Equal(t, 2, ComputeIfAbsent(m, 1, func(k int) int {
		return -k
	}))
	assert.Equal(t, 2, m[1])
	assert.Equal(t, -4, ComputeIfAbsent(m, 4, func(k int) int {
		return -k
	}))
	assert.Equal(t, -4, m[4])
}

func TestKeysValues(t *testing.T) {
	m := map[int]int{
		1: 2,
		2: 4,
		3: 6,
	}

	keys := set.New(Keys(m)...)
	assert.True(t, set.Equals(set.New(1, 2, 3), keys))
	values := Values(m)
	slicex.Sort(values)
	assert.Equal(t, []int{2, 4, 6}, values)
}

func TestConvertValues(t *testing.T) {
	m := map[int]int{
		1: 2,
		2: 4,
		3: 6,
	}
	nm := ConvertValues(m, func(v int) string {
		return intx.Format(v)
	})
	assert.Equal(t, 3, len(nm))
	assert.Equal(t, "2", nm[1])
}

func TestCopy(t *testing.T) {
	m := map[int]int{
		1: 2,
		2: 4,
		3: 6,
	}
	nm := Copy(m)
	assert.Equal(t, m, nm)
	m[1] = 3
	assert.Equal(t, 2, nm[1])
}

func TestContainsAnyKey(t *testing.T) {
	m := map[int]int{
		1: 2,
		2: 4,
		3: 6,
	}
	assert.True(t, ContainsAnyKey(m, 1, -1))
	assert.False(t, ContainsAnyKey(m, -1))
	assert.False(t, ContainsAnyKey(m))
}
