package set

import (
	"testing"

	"github.com/hsiafan/glow/v2/container/slicex"
	"github.com/stretchr/testify/assert"
)

func TestSet(t *testing.T) {
	s := New[int]()
	s.Add(1)
	s.AddAll(1, 2, 3)
	assert.True(t, s.Contains(1))
	assert.True(t, s.Contains(2))
	s.Remove(1)
	assert.False(t, s.Contains(1))
	assert.Equal(t, 2, s.Size())
	ss := s.ToSlice()
	slicex.Sort(ss)
	assert.Equal(t, []int{2, 3}, ss)
}

func TestSet_Add(t *testing.T) {
	s := New[int]()
	s2 := New[int]()
	s.Add(1)
	s2.AddAll(2, 3)
	s.AddSet(s2)
	ss := s.ToSlice()
	slicex.Sort(ss)
	assert.Equal(t, []int{1, 2, 3}, ss)
}

func TestSet_ContainsAny(t *testing.T) {
	s := New(1, 2, 3)
	assert.True(t, s.ContainsAny(3, 4, 5))
	assert.False(t, s.ContainsAny())
	assert.False(t, s.ContainsAny(0, -1))
}

func TestIntersect(t *testing.T) {
	s1 := New(1, 2, 3)
	s2 := New(0, 2, 4)
	s3 := New(0, 4)
	assert.True(t, Equals(New(2), Intersect(s1, s2)))
	assert.True(t, Intersect(s1, s3).Size() == 0)
}

func TestUnion(t *testing.T) {
	s1 := New(1, 2, 3)
	s2 := New(0, 2, 4)
	s3 := Set[int]{}
	assert.True(t, Equals(New(0, 1, 2, 3, 4), Union(s1, s2)))
	assert.True(t, Equals(s1, Union(s1, s3)))
}

func TestDifference(t *testing.T) {
	s1 := New(1, 2, 3)
	s2 := New(0, 2, 4)
	s3 := Set[int]{}
	assert.True(t, Equals(New(1, 3), Difference(s1, s2)))
	assert.True(t, Equals(New(0, 4), Difference(s2, s1)))
	assert.True(t, Equals(s1, Difference(s1, s3)))
}

func TestSymmetricDifference(t *testing.T) {
	s1 := New(1, 2, 3)
	s2 := New(0, 2, 4)
	s3 := Set[int]{}
	assert.True(t, Equals(New(1, 3, 0, 4), SymmetricDifference(s1, s2)))
	assert.True(t, Equals(s1, SymmetricDifference(s1, s3)))
}
