package slicex

import (
	"testing"

	"github.com/hsiafan/glow/v2/mathx/intx"
	"github.com/stretchr/testify/assert"
)

func TestInsertAt(t *testing.T) {
	var s = []int{1, 2}
	s = Insert(s, 1, 3)
	assert.Equal(t, []int{1, 3, 2}, s)
	s = Insert(s, 0, 1)
	assert.Equal(t, []int{1, 1, 3, 2}, s)
	s = Insert(s, 4, -1)
	assert.Equal(t, []int{1, 1, 3, 2, -1}, s)

	s = []int{}
	s = Insert(s, 0, 1)
	assert.Equal(t, []int{1}, s)
}

func TestRemoveAt(t *testing.T) {
	var s = []int{1, 2, 3, 4, 5}
	s = RemoveAt(s, 4)
	assert.Equal(t, []int{1, 2, 3, 4}, s)
	s = RemoveAt(s, 1)
	assert.Equal(t, []int{1, 3, 4}, s)
	s = RemoveAt(s, 0)
	assert.Equal(t, []int{3, 4}, s)
	s = RemoveAt(s, 0)
	assert.Equal(t, []int{4}, s)
	s = RemoveAt(s, 0)
	assert.Equal(t, []int{}, s)
}

func TestRemoveWhen(t *testing.T) {
	var s = []int{1, 2, 3, 4, 5}
	s = RemoveBy(s, func(value int) bool {
		return value%2 == 0
	})
	assert.Equal(t, []int{1, 3, 5}, s)
}

func TestConvert(t *testing.T) {
	var s = []int{1, 2, 3}
	result := Convert(s, func(s int) string {
		return intx.Format(s)
	})
	assert.Equal(t, []string{"1", "2", "3"}, result)
}

func TestSelect(t *testing.T) {
	var s = []int{1, 2, 3}
	var result = Select(s, func(v int) bool {
		return v%2 == 0
	})
	assert.Equal(t, []int{2}, result)
}

func TestFind(t *testing.T) {
	var s = []int{1, 2, 3}
	assert.Equal(t, 1, Find(s, 2))
	assert.Equal(t, -1, Find(s, 10))
}

func TestFindLast(t *testing.T) {
	var s = []int{1, 2, 3}
	assert.Equal(t, 0, FindLast(s, 1))
	assert.Equal(t, -1, FindLast(s, 10))
}

func TestSort(t *testing.T) {
	var s = []int{1, 3, 2}
	Sort(s)
	assert.Equal(t, []int{1, 2, 3}, s)
}

func TestCopy(t *testing.T) {
	var s = []int{1, 2, 3}
	ts := Copy(s)
	assert.Equal(t, s, ts)
	s[0] = 2
	assert.Equal(t, 1, ts[0])
}

func TestPartition(t *testing.T) {
	var s = []int{1, 2, 3}
	matched, mismatched := Partition(s, func(v int) bool {
		return v%2 == 0
	})
	assert.Equal(t, []int{2}, matched)
	assert.Equal(t, []int{1, 3}, mismatched)
}

func TestSortBy(t *testing.T) {
	type Ele struct {
		value int
		name  string
	}
	var s = []*Ele{
		{name: "test3", value: 3},
		{name: "test1", value: 1},
		{name: "test4", value: 4},
		{name: "test2", value: 2},
	}
	SortBy(s, func(v1, v2 *Ele) bool {
		return v1.value < v2.value
	})
	assert.Equal(t, "test1", s[0].name)
	assert.Equal(t, 1, s[0].value)
}

func TestMaxMin(t *testing.T) {
	s := []int{1, 2, 3}
	assert.Equal(t, 3, Max(s).Must())
	assert.Equal(t, 1, Min(s).Must())
}

func TestFirst(t *testing.T) {
	s := []int{1, 2, 3}
	var s2 []int
	assert.Equal(t, 1, First(s).Must())
	assert.False(t, First(s2).Present())
}

func TestLast(t *testing.T) {
	s := []int{1, 2, 3}
	var s2 []int
	assert.Equal(t, 3, Last(s).Must())
	assert.False(t, Last(s2).Present())
}

func TestPrepend(t *testing.T) {
	assert.Equal(t, []int{0, 1, 2, 3}, Prepend([]int{1, 2, 3}, 0))
	assert.Equal(t, []int{0, 1, 2}, Prepend([]int{1, 2, 3}[:2], 0))
	assert.Equal(t, []int{0}, Prepend([]int{}, 0))
	assert.Equal(t, []int{0}, Prepend(nil, 0))
}

func TestAppend(t *testing.T) {
	assert.Equal(t, []int{1, 2, 3}, Append([]int{1, 2}, 3))
}

func TestConcat(t *testing.T) {
	assert.Equal(t, []int{1, 2, 3}, Concat([]int{1}, []int{2, 3}))
	assert.Equal(t, []int{1, 2, 3}, Concat([]int{1}, nil, []int{2, 3}, []int{}))
}
