package jsonx

import (
	"bytes"
	"encoding/json"

	"github.com/hsiafan/glow/v2/stringx"
)

// Marshal encode value to json string, without html escape
func Marshal(v interface{}) (string, error) {
	var buf bytes.Buffer
	enc := json.NewEncoder(&buf)
	enc.SetEscapeHTML(false)
	if err := enc.Encode(v); err != nil {
		return "", err
	}
	data := buf.Bytes()
	l := len(data)
	if l > 0 && data[l-1] == '\n' {
		data = data[:l-1]
	}
	return stringx.WrapBytes(data), nil
}

// MarshalIndent is like Marshal but applies Indent to format the output.
// Each JSON element in the output will begin on a new line beginning with prefix
// followed by one or more copies of indent according to the indentation nesting.
func MarshalIndent(v interface{}, indent string) (string, error) {
	var buf bytes.Buffer
	enc := json.NewEncoder(&buf)
	enc.SetEscapeHTML(false)
	enc.SetIndent("", indent)
	if err := enc.Encode(v); err != nil {
		return "", err
	}
	data := buf.Bytes()
	l := len(data)
	if l > 0 && data[l-1] == '\n' {
		data = data[:l-1]
	}
	return stringx.WrapBytes(data), nil
}

// UnMarshal parse json encoded string to value, return the value.
func UnMarshal[T any](jsonStr string) (T, error) {
	var t T
	err := json.Unmarshal(stringx.AsBytes(jsonStr), &t)
	return t, err
}

// UnMarshalPtr parse json encoded string to value, return the pointer to value.
func UnMarshalPtr[T any](jsonStr string) (*T, error) {
	var t T
	err := json.Unmarshal(stringx.AsBytes(jsonStr), &t)
	return &t, err
}
