package flagx

import (
	"fmt"
	"reflect"
	"strconv"
	"strings"

	"github.com/hsiafan/glow/v2/stringx/ascii"
)

func toFlagName(fieldName string) string {
	var sb strings.Builder

	for i := 0; i < len(fieldName); i++ {
		c := fieldName[i]
		if ascii.IsUpper(c) {
			if sb.Len() != 0 {
				sb.WriteByte('-')
			}
			sb.WriteByte(ascii.ToLower(c))
		} else {
			sb.WriteByte(c)
		}
	}
	return sb.String()
}

// mustGetBoolTagValue get tag value as boolean with name. if not present, return defaultValue.
// If tag present and parse error, return error.
func mustGetBoolTagValue(field reflect.StructField, tagName string, defaultValue bool) bool {
	if value, ok := field.Tag.Lookup(tagName); ok {
		result, err := strconv.ParseBool(value)
		if err != nil {
			panic(fmt.Errorf("struct tag %s:%s parse error, %w", field.Name, tagName, err))
		}
		return result
	}
	return defaultValue
}
