package flagx

import (
	"testing"
	"time"

	"github.com/hsiafan/glow/v2/timex/durationx"

	"github.com/stretchr/testify/assert"
)

func TestNewCommandLine(t *testing.T) {
	cmd := NewCommand("my", "", func(op *Option) error {
		assert.True(t, op.Update)
		assert.False(t, op.Dry)
		assert.Equal(t, "kite", op.Name)
		assert.Equal(t, []int{1, 2}, op.Tags)
		assert.Equal(t, []string{"4s"}, op.Args)
		assert.Equal(t, "file1", op.File)
		assert.Equal(t, 1, op.Age)
		assert.Equal(t, durationx.OfMinutes(1), op.Timeout)
		assert.Equal(t, durationx.OfSeconds(3), op.Timeout2)
		return nil
	})
	cmd.Description = "a test cmd"
	cmd.ParseAndExecute([]string{"-update", "-dry=false", "-name", "kite", "-timeout", "1m", "-tags", "1",
		"-tags", "2", "file1", "3s", "4s"})

	//cmd.ShowUsage()
}

type Option struct {
	Update  bool          `flag:""`
	Dry     bool          `flag:"" default:"true"`
	Name    string        `flag:"" description:"the name"`
	Age     int           `flag:"age" description:"the age" default:"1"`
	Tags    []int         `flag:""`
	Timeout time.Duration `flag:"" default:"0"`
	Args    []string      `argument:""`
	InOption
}

type InOption struct {
	File     string        `argument:"0"`
	Timeout2 time.Duration `argument:"1"`
}
