package gox

import (
	"unsafe"

	"github.com/hsiafan/glow/v2/gox/constraints"
)

// IsType return true if an interface is type T
func IsType[T any](value any) bool {
	_, ok := value.(T)
	return ok
}

// Zero return zero value for type T
func Zero[T any]() (value T) {
	return
}

// SizeOf return size of type T
func SizeOf[T any]() uintptr {
	var zero T
	return unsafe.Sizeof(zero)
}

// Max return the max value
func Max[T constraints.Ordered](v1, v2 T) T {
	if v1 < v2 {
		return v2
	}
	return v1
}

// Min return the min value
func Min[T constraints.Ordered](v1, v2 T) T {
	if v1 > v2 {
		return v2
	}
	return v1
}

// ForceCast force cast value to another type. The two type must have same size.
func ForceCast[S any, T any](v S) T {
	if unsafe.Sizeof(v) != SizeOf[T]() {
		panic("force cast type has different size")
	}

	return *((*T)(unsafe.Pointer(&v)))
}
