package gox

import (
	"errors"
	"testing"
	"unsafe"

	"github.com/stretchr/testify/assert"
)

func TestIsType(t *testing.T) {
	assert.True(t, IsType[int](1))

	var e = errors.New("1")
	assert.True(t, IsType[error](e))
	assert.False(t, IsType[int](e))
	assert.False(t, IsType[string](e))

	var s = ""
	assert.True(t, IsType[string](s))
}

func TestZero(t *testing.T) {
	assert.Equal(t, 0, Zero[int]())
	assert.Equal(t, "", Zero[string]())
	assert.Equal(t, (*int)(nil), Zero[*int]())
	assert.Equal(t, nil, Zero[interface{}]())
}

func TestMax(t *testing.T) {
	assert.Equal(t, 2, Max(1, 2))
	assert.Equal(t, "2", Max("1", "2"))
}

func TestMin(t *testing.T) {
	assert.Equal(t, 1, Min(1, 2))
	assert.Equal(t, "1", Min("1", "2"))
}

func TestForceCast(t *testing.T) {
	m := map[int]string{}
	mp := ForceCast[map[int]string, unsafe.Pointer](m)
	assert.NotZero(t, mp)

	f := func() {
		_ = m
	}
	fp := ForceCast[func(), unsafe.Pointer](f)
	assert.NotZero(t, fp)

	assert.Panics(t, func() {
		ForceCast[bool, unsafe.Pointer](false)
	})
}

func TestSizeOf(t *testing.T) {
	assert.Equal(t, uintptr(8), SizeOf[int64]())
	assert.Equal(t, uintptr(24), SizeOf[[]int]())
	assert.Equal(t, uintptr(8), SizeOf[map[int]int]())
	assert.Equal(t, uintptr(1), SizeOf[bool]())
}
