package must

import "errors"

// NoError is used for checking func which returns an error, and no return value.
// If err is not nil, panics with the error.
func NoError(err error) {
	if err != nil {
		panic(err)
	}
}

// NoError1 is used for checking func which returns a value and an error.
// If err is not nil, panics with the error.
func NoError1[T any](value T, err error) T {
	if err != nil {
		panic(err)
	}
	return value
}

// NoError2 is used for checking func which returns two value and an error.
// If err is not nil, panics with the error.
func NoError2[S any, T any](value1 S, value2 T, err error) (S, T) {
	if err != nil {
		panic(err)
	}
	return value1, value2
}

var notOkError = errors.New("not ok, value not present")

// OK return value when ok is true; panics otherwise.
// This method is for check optional exists values.
func OK[T any](value T, ok bool) T {
	if !ok {
		panic(notOkError)
	}
	return value
}
