package option

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestOptional_NotPresent(t *testing.T) {
	o := Empty[int]()
	assert.Panics(t, func() {
		o.Must()
	})
	assert.False(t, o.Present())
	assert.Equal(t, 1, o.OrElse(1))
	assert.Equal(t, 2, o.OrCompute(func() int {
		return 2
	}))
}

func TestOptional_Present(t *testing.T) {
	o := OfValue(1)
	assert.Equal(t, 1, o.Must())
	assert.True(t, o.Present())
	assert.Equal(t, 1, o.OrElse(2))
	assert.Equal(t, 1, o.OrCompute(func() int {
		return 2
	}))
}

func TestOptional_OrNext(t *testing.T) {
	v := Empty[int]().OrNext(func() Option[int] {
		return Empty[int]()
	}).OrCompute(func() int {
		return 2
	})
	assert.Equal(t, 2, v)

	v = Empty[int]().OrNext(func() Option[int] {
		return OfValue(1)
	}).OrElse(2)
	assert.Equal(t, 1, v)

	v = OfValue(1).OrNext(func() Option[int] {
		return Empty[int]()
	}).OrElse(2)
	assert.Equal(t, 1, v)
}
