package floatx

import (
	"math"
	"strconv"
	"unsafe"

	"github.com/hsiafan/glow/v2/gox/constraints"
)

// Format convert float to string, with decimals numbers.
func Format[T constraints.Float](value T, decimals int) string {
	return strconv.FormatFloat(float64(value), 'f', decimals, int(unsafe.Sizeof(value))*8)
}

// SimpleFormat convert float to string
func SimpleFormat[T constraints.Float](value T) string {
	return strconv.FormatFloat(float64(value), 'f', -1, int(unsafe.Sizeof(value))*8)
}

// Parse convert str to float.
func Parse[T constraints.Float](str string) (T, error) {
	var t T
	value, err := strconv.ParseFloat(str, int(unsafe.Sizeof(t))*8)
	if err != nil {
		return t, err
	}
	return T(value), err
}

// SafeParse convert str to float. if str is not a illegal uint value representation, return defaultValue
func SafeParse[T constraints.Float](str string, defaultValue T) T {
	if value, err := Parse[T](str); err == nil {
		return value
	}
	return defaultValue
}

// Max return the larger one of float values
func Max[T constraints.Float](value1, value2 T) T {
	return T(math.Max(float64(value1), float64(value2)))
}

// Min return the smaller one of float values
func Min[T constraints.Float](value1, value2 T) T {
	return T(math.Min(float64(value1), float64(value2)))
}
