package intx

import (
	"strconv"
	"unsafe"

	"github.com/hsiafan/glow/v2/gox/constraints"
)

const MaxInt = int(MaxUint >> 1)
const MinInt = -MaxInt - 1
const MaxUint = ^uint(0)
const MinUint = 0

// Format convert int to str
func Format[T constraints.Signed](v T) string {
	return strconv.FormatInt(int64(v), 10)
}

// Parse convert str to int
func Parse[T constraints.Signed](str string) (T, error) {
	var t T
	value, err := strconv.ParseInt(str, 10, int(unsafe.Sizeof(t))*8)
	if err != nil {
		return t, err
	}
	return T(value), err
}

// SafeParse convert str to int. if str is not a illegal int value representation, return defaultValue
func SafeParse[T constraints.Signed](str string, defaultValue T) T {
	value, err := strconv.ParseInt(str, 10, int(unsafe.Sizeof(defaultValue))*8)
	if err == nil {
		return T(value)
	}
	return defaultValue
}

// FormatUnsigned convert uint to str
func FormatUnsigned[T constraints.Unsigned](v T) string {
	return strconv.FormatUint(uint64(v), 10)
}

// ParseUnsigned convert str to uint. if str is not a illegal uint value representation, return defaultValue
func ParseUnsigned[T constraints.Unsigned](str string) (T, error) {
	var i T
	v, err := strconv.ParseUint(str, 10, int(unsafe.Sizeof(i))*8)
	return T(v), err
}

// SafeParseUnsigned convert str to uint. if str is not a illegal uint value representation, return defaultValue
func SafeParseUnsigned[T constraints.Unsigned](str string, defaultValue T) T {
	v, err := strconv.ParseUint(str, 10, int(unsafe.Sizeof(defaultValue))*8)
	if err == nil {
		return T(v)
	}
	return defaultValue
}

// Abs return absolute value for int.
// Note: Abs(MinInt) return MinInt self
func Abs[T constraints.Signed](value T) T {
	if value < 0 {
		return -value
	}
	return value
}

// Min return smaller one of int values
func Min[T constraints.Integer](value1, value2 T) T {
	if value1 < value2 {
		return value1
	}
	return value2
}

// Max return bigger one of int values
func Max[T constraints.Integer](value1, value2 T) T {
	if value1 > value2 {
		return value1
	}
	return value2
}
