package stringx

import "strings"

// Splitter is a tool to split string with prefix, suffix, and separator into pieces.
//
// Usage:
//  splitter := &Splitter{Separator:",", Prefix:"[", Suffix:"]"}
//  ss := splitter.Split(str)
type Splitter struct {
	Prefix    string // the prefix of string to remove
	Suffix    string // the suffix of string to remove
	Separator string // the delimiter to split str
}

// Split spit a string into pieces, using prefix, suffix, and separator of this Splitter.
func (sp *Splitter) Split(s string) []string {
	if sp.Prefix != "" && strings.HasPrefix(s, sp.Prefix) {
		s = s[len(sp.Prefix):]
	}
	if sp.Suffix != "" && strings.HasSuffix(s, sp.Suffix) {
		s = s[:len(s)-len(sp.Suffix)]
	}
	return strings.Split(s, sp.Separator)
}
