package stringx

import (
	"unsafe"
)

// sliceHeader copy from reflect.SliceHeader, but using Data type unsafe.Pointer instead of unitptr
type sliceHeader struct {
	Data unsafe.Pointer
	Len  int
	Cap  int
}

// stringHeader copy from reflect.StringHeader, but using Data type unsafe.Pointer instead of unitptr
type stringHeader struct {
	Data unsafe.Pointer
	Len  int
}

// WrapBytes unsafe convert byte array content to string.
// This function do not allocate new buffer, reuse the byte array.
func WrapBytes(bytes []byte) string {
	sliceHeader := (*sliceHeader)(unsafe.Pointer(&bytes))
	stringHeader := stringHeader{
		Data: sliceHeader.Data,
		Len:  sliceHeader.Len,
	}
	sp := (*string)(unsafe.Pointer(&stringHeader))
	return *sp
}

// AsBytes unsafe convert string to byte slice.
// This function do not allocate new buffer, reuse the string buffer.
func AsBytes(str string) []byte {
	stringHeader := (*stringHeader)(unsafe.Pointer(&str))
	sliceHeader := sliceHeader{
		Data: stringHeader.Data,
		Len:  stringHeader.Len,
		Cap:  stringHeader.Len, // just same with len
	}
	sp := (*[]byte)(unsafe.Pointer(&sliceHeader))
	return *sp
}
